<!DOCTYPE html>
<html lang="en-US">
{% set title = "Login" %}
{% include '_head.html' %}

<body>
    <div class="content">
        <div class="center-block">
            <p>MLFlow authentication:</p>
            <p><a href="{{ url_for('login') }}"><button>{{ provide_display_name }}</button></a></p>
        </div>
    </div>
    {% include '_footer.html' %}
</body>

</html>